package com.example.http_dns;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import androidx.annotation.NonNull;
import com.alibaba.sdk.android.httpdns.HttpDns;
import com.alibaba.sdk.android.httpdns.HttpDnsService;
import com.alibaba.sdk.android.httpdns.probe.IPProbeItem;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * HttpDnsPlugin
 */
public class HttpDnsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private static HttpDnsService httpdns;
    private static final String APPLE_URL = "ottoapi.niuniutaojin.com";
    private MethodChannel channel;
    private FlutterPluginBinding pluginBinding;
    private ActivityPluginBinding activityBinding;
    private static ExecutorService pool = Executors.newSingleThreadExecutor();
    private static final String TAG = "HttpDnsPlugin";
    private static final String CHANNEL_NAME = "http_dns";

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("setup")) {
            setup(call, result);
        } else if (call.method.equals("getIpByUrl")) {
            getIpByUrl(call, result);
        } else if (call.method.equals("getIpsByUrl")) {
            getIpsByUrl(call, result);
        } else {
            result.notImplemented();
        }
    }


    private void setup(@NonNull MethodCall call, @NonNull Result result) {
        Log.d(TAG, "setup :" + call.arguments);
        Map<String, Object> resultMap = new HashMap<>();
        HashMap<String, Object> map = call.arguments();

        // 初始化httpdns
        if (call.hasArgument("accountID")) {
            if (call.hasArgument("secretKey") && map.get("secretKey") != null) {
                httpdns = HttpDns.getService(pluginBinding.getApplicationContext(), map.get("accountID").toString(), map.get("secretKey").toString());
            } else {
                httpdns = HttpDns.getService(pluginBinding.getApplicationContext(), map.get("accountID").toString());
            }
        } else {
            result.success("not found accountID");
            throw new RuntimeException("not found accountID");
        }

        // 可以替换成您在后台配置的域名
        if (call.hasArgument("preResolveHosts")) {
            ArrayList<String> arrayList = new ArrayList<String>();
            arrayList.addAll((List) map.get("preResolveHosts"));
            httpdns.setPreResolveHosts(arrayList);

            //httpdns为初始化获取的服务实例
            ArrayList<IPProbeItem> list = new ArrayList<IPProbeItem>();
            for (int i = 0; i < arrayList.size(); i++) {
                list.add(new IPProbeItem(arrayList.get(i), 8080));
            }
            httpdns.setIPProbeList(list);
        }

        // 允许过期IP以实现懒加载策略
        if (call.hasArgument("expiredIPEnabled")) {
            httpdns.setExpiredIPEnabled((boolean) map.get("expiredIPEnabled"));
        }

        // 打开HTTPDNS Log，线上建议关闭
        if (call.hasArgument("logEnabled")) {
            httpdns.setLogEnabled((boolean) map.get("logEnabled"));
        }

        result.success("ok");
    }

    private void getIpByUrl(@NonNull MethodCall call, final Result result) {
        final String str = call.arguments();
        pool.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    String originalUrl = str;
                    URL url = new URL(originalUrl);
                    // 同步接口获取IP
                    String ip = httpdns.getIpByHostAsync(url.getHost());
                    if (ip != null) {
                        // 通过HTTPDNS获取IP成功
                        Log.d(TAG, "GetIP: " + ip.toString() + " for host: " + url.getHost() + " from HTTPDNS successfully!");
                    }
                    runMainThreadByIp(ip, result);
                } catch (Exception e) {
                    runMainThreadByIp(null, result);
                    e.printStackTrace();
                }
            }
        });
    }

    private void getIpsByUrl(@NonNull MethodCall call, final Result result) {
        final String str = call.arguments();
        pool.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    String originalUrl = str;
                    URL url = new URL(originalUrl);
                    // 同步接口获取IP
                    String[] ips = httpdns.getIpsByHostAsync(url.getHost());
                    if (ips != null) {
                        // 通过HTTPDNS获取IP成功
                        Log.d(TAG, "GetIPS: " + ips.toString() + " for host: " + url.getHost() + " from HTTPDNS successfully!");
                    }
                    runMainThreadByIps(ips, result);
                } catch (Exception e) {
                    runMainThreadByIps(null, result);
                    e.printStackTrace();
                }
            }
        });
    }

    // 主线程再返回数据
    public void runMainThreadByIp(final String ip, final Result result) {
        Handler handler = new Handler(Looper.getMainLooper());
        handler.post(new Runnable() {
            @Override
            public void run() {
                result.success(ip);
            }
        });
    }

    // 主线程再返回数据
    public void runMainThreadByIps(final String[] ips, final Result result) {
        final ArrayList<String> ipList = new ArrayList<String>(Arrays.asList(ips));
        Handler handler = new Handler(Looper.getMainLooper());
        handler.post(new Runnable() {
            @Override
            public void run() {
                result.success(ipList);
            }
        });
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {

        pluginBinding = binding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        pluginBinding = null;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        channel = new MethodChannel(pluginBinding.getBinaryMessenger(), CHANNEL_NAME);
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
}
